//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2003, The Institute for Genomic Research (TIGR), Rockville,
// Maryland, U.S.A.  All rights reserved.
//////////////////////////////////////////////////////////////////////////////
#include "DataDescrip.h"
#include "Prediction.h"
#include <fstream>
#include <assert.h>

void DataDescrip::add(EvidenceType* ev) {
  _evMap[strdup(ev->getStr())] = ev;
  ++_size;
}

void DataDescrip::initVec(FeatVec& vec) const {
  evHashMap::const_iterator iter = _evMap.begin();
  evHashMap::const_iterator stop = _evMap.end();
  while( iter != stop ) {
    const EvidenceType* ev = (*iter).second;
    vec[ev->getIdx()] = ev->dup();
    ++iter;
  }
}

int DataDescrip::getNumDim() const {
  evHashMap::const_iterator iter = _evMap.begin();
  evHashMap::const_iterator stop = _evMap.end();
  int cnt = 0;
  while( iter != stop ) {
    const EvidenceType* ev = (*iter).second;
    cnt += ev->getNum();
    ++iter;
  }
  return cnt;
}

FeatVec* DataDescrip::read(std::istream& ifs) const {
  FeatVec* retVec = new FeatVec(size());
  initVec(*retVec);
  FeatVec& vec = *retVec;
  evHashMap::const_iterator iter = _evMap.begin();
  evHashMap::const_iterator stop = _evMap.end();
  int cnt = 0;
  while( iter != stop ) {
    EvidenceType& et = *(vec[cnt]);
    assert(et.getIdx()==cnt);
    if( !et.read(ifs) ) {
      delete retVec;
      return NULL;
    }
    ++iter;
    ++cnt;
  }
  return retVec;
}

int DataDescrip::getIdx(const Prediction* pred) const {
  const char* id = pred->getDataSource().c_str();
  evHashMap::const_iterator iter = _evMap.find(id);
  if(iter == _evMap.end()) {
    return -1;
  }
  const EvidenceType* ev = (*iter).second; 
  return ev->getIdx();
}
